package main

import (
	"crypto/md5"
	"encoding/hex"
	"net/url"
	"bytes"
	"fmt"
	"sort"
    "log"
    "net/http"
	"io/ioutil"
	"encoding/json"
	"strings"
)

func main() {
    http.HandleFunc("/", handler)
    http.ListenAndServe(":8082", nil)
}

func handler(w http.ResponseWriter, r *http.Request) {
	
	w.Header().Set("Access-Control-Allow-Origin", r.Header.Get("Origin"))
	w.Header().Set("Access-Control-Allow-Methods", "POST, GET, OPTIONS, PUT, DELETE")
	
	err := r.ParseForm()
	if err != nil {
		w.Write([]byte(err.Error()))
		return
	}

	//fmt.Println(r.PostForm)
	
	sender := r.PostForm["sender"][0]
	text := r.PostForm["text_message"][0]
	send_type := r.PostForm["send_type"][0]
	phonesA := strings.Split(r.PostForm["phones"][0],"\r\n")
	phones := make([][]string, len(phonesA))
	for key, values := range phonesA {
		phones[key] = make([]string, 1)
		phones[key][0] = values;
	}
	
	result, err := sendSMS(sender, text, phones, send_type)
	if err != nil {
		panic(err)
	}

	var dat map[string]interface{}
	bytes2 := []byte(result)
	if err := json.Unmarshal(bytes2, &dat); err != nil {
		panic(err)
	}

	w.Header().Set("Content-Type", "application/json; charset=UTF-8")
	w.WriteHeader(http.StatusOK)
	if err := json.NewEncoder(w).Encode(dat); err != nil {
		panic(err)
	}
}

func sendSMS(sender string, text string, phones [][]string, send_type string) (string, error) {

	public_key := "PUBLIC_KEY"
	private_key := "PRIVATE_KEY"

	version := "3.0"
	action := "sendSMSGroup"
	action = strings.ToLower(action)
	datetime := ""
	if (send_type == "now") {
		datetime = ""
	}
	sms_lifetime := "0"
	batch := "0"
	batchinterval := "0"
	
	control_str := ""
	phonesJ, err := json.Marshal(phones)
	if err != nil {
		panic(err)
	}
	
	params := make(map[string]string)
	params["sender"] = sender
	params["text"] = text
	params["phones"] = string(phonesJ)
	params["datetime"] = datetime
	params["sms_lifetime"] = sms_lifetime
	params["batch"] = batch
	params["batchinterval"] = batchinterval
	params["key"] = public_key
    params["version"] = version
    params["action"] = action

	keys := make([]string, 0, len(params))
	for k := range params {
		keys = append(keys, k)
	}
	sort.Strings(keys)
	for _, k := range keys {
		control_str += params[k]
	}
	control_str += private_key
	
	result := ""

	// log.Printf("%s\n", control_str)
	
	hasher := md5.New()
	hasher.Write([]byte(control_str))
	control_sum := hex.EncodeToString(hasher.Sum(nil))
	
	// log.Printf("%s\n", control_sum)

	form := url.Values{
		"sender":       {sender},
		"text":         {text},
		"phones":       {string(phonesJ)},
		"datetime":     {datetime},
		"batch":		{batch},
		"batchinterval":{batchinterval},
		"key":          {public_key},
		"sum":          {control_sum},
		"sms_lifetime": {sms_lifetime},
	}
	
	body := bytes.NewBufferString(form.Encode())
	url := fmt.Sprintf("http://api.atompark.com/api/sms/3.0/" + action)

	response, err := http.Post(url, "application/x-www-form-urlencoded", body)
	if err != nil {
		// fmt.Printf("%s", err)
		return result, err
	} else {
		defer response.Body.Close()
		contents, err := ioutil.ReadAll(response.Body)
		if err != nil {
			// fmt.Printf("%s", err)
			return result, err
		}
		// fmt.Printf("%s\n", string(contents))
		return string(contents), nil
	}
    return result, nil
}